import React, { useState, useMemo, useEffect } from 'react'
import { Select, message, Tag } from 'antd'
import { GetSelectItemList, GetFieldValueFromTable, getStationListByUserID, getStationIDListByUserID } from '../../../../apis/process'
import TagPack from '../../../components/TagPack'

const { Option } = Select

const ComboBox = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { value, onChange, schema, addons } = props
  const {
    placeholder,
    disabled,
    formDisabled,
    sourceType,
    options,
    dictionary,
    tableName,
    fieldName,
    presetValue,
    isMultiple,
    isEdit,
    isMySite,
    saveVal,
    isSearch,
    isStoreID,
    color,
    siteSelect
  } = schema
  const [dictionaryList, setDictionaryList] = useState([])
  const [tableData, setTableData] = useState([])
  const [site, setSite] = useState([])
  useEffect(() => {
    console.log(props, 'propssssss')
    if (addons) {
      if (sourceType === '站点') {
        getStation(presetValue)
      }
      if(sourceType === '数据字典' && dictionary && saveVal){
        getDictionaryItemList(dictionary)
      }else{
        addons.setValue(addons.dataPath, presetValue || '')
      }
    } else {
      onChange(presetValue || '')
    }
  }, [presetValue, sourceType, isMultiple])

  const valueShow = useMemo(() => {
    if (isMultiple || isEdit) {
      return value ? value.split(',') : []
    } else {
      return value || null
    }
  }, [isEdit, isMultiple, value])

  const enums = useMemo(() => {
    switch (sourceType) {
      case '手动输入':
        return Array.isArray(options) ? options : []
      case '数据字典':
        return Array.isArray(dictionaryList) ? saveVal ? dictionaryList : dictionaryList.map(v=> v.nodeName) : []
      case '表数据':
        return Array.isArray(tableData) ? tableData : []
      case '站点':
        return Array.isArray(site) ? site : []
      default:
        return []
    }
  }, [sourceType, options, dictionaryList, tableData, site])

  const children = useMemo(() => {
    let children = []
    if (Array.isArray(enums)) {
      enums.forEach(v => {
        if (sourceType === '站点') {
          children.push(<Option key={isStoreID ? v.stationID : v.stationName} value={isStoreID ? v.stationID : v.stationName}>{v.stationName}</Option>)
        } else if (sourceType === '手动输入') {
          children.push(
            <Option key={v.value} value={v.value}>
              {color ? <TagPack color={v.color} text={v.value} /> : v.value}
            </Option>
          )
        } else {
          if(sourceType === '数据字典' && saveVal){
            children.push(<Option key={v} value={v.nodeValue}>{v.nodeName}</Option>)
          }else{
            children.push(<Option key={v} value={v}>{v}</Option>)
          }
        }
      })
    }
    return children
  }, [sourceType, enums, isStoreID])

  //为了支持可编辑形态单选
  const handleChange = (value, option) => {
    if (addons) {
      if (isMultiple) {
        onChange(value.join(',') || '')
      } else if (!isMultiple && isEdit) {
        return
      } else {
        onChange(value || '')
      }
    }
  }

  const onSelect = (selectValue) => {
    if (!isMultiple && isEdit) {
      if (selectValue !== value) {
        onChange(selectValue)
      }
    }
  }

  const onDeselect = (value) => {
    if (!isMultiple && isEdit) {
      onChange('')
    }
  }

  const onClear = () => {
    onChange('')
  }

  const getStation = async (presetValue) => {
    const { code, data, msg } = await getStationIDListByUserID(userID, !isMySite)
    if (code === 0) {
      if (Array.isArray(data)) {
        let _data = data;
        if(sourceType === '站点' && siteSelect && siteSelect.length > 0){
          _data = _data.filter(s=> siteSelect.includes(s.stationID))
        }
        let values = _data.map(v => ({ stationID: v.stationID + '', stationName: v.stationName }))
        setSite(values)
        if (!value && !presetValue && !formDisabled) {
          onChange(values[0][isStoreID ? 'stationID' : 'stationName'])
        }
      } else {
        setSite([])
      }
    } else {
      message.error(msg)
    }
  }

  const getDictionaryItemList = async (dictionary)=>{
    if (!dictionary) {
      return message.error('请选择数据字典！')
    }
    const { data } = await GetSelectItemList({ nodeName: dictionary })
    if (Array.isArray(data)) {
      setDictionaryList(data)
      if(saveVal && presetValue){
        const d = data.find(v=> v.nodeName === presetValue)
        if(d){
          addons.setValue(addons.dataPath, d.nodeValue)
        }
      }
    }
  }

  const onFocus = async () => {
    if (addons) {
      if (sourceType === '数据字典') {
        if (!dictionary) {
          return message.error('请选择数据字典！')
        }
        if(dictionaryList.length > 0){
          return;
        }
        const { data } = await GetSelectItemList({ nodeName: dictionary })
        if (Array.isArray(data)) {
          setDictionaryList(data)
        }
      }
      if (sourceType === '表数据') {
        if (!tableName) {
          return message.info('请选择表名!')
        }
        if (!fieldName) {
          return message.info('请选择字段名!')
        }
        const { code, data, msg } = await GetFieldValueFromTable(tableName, fieldName)
        if (code === 0) {
          if (Array.isArray(data)) {
            setTableData(data.filter(v => v))
          } else {
            setTableData([])
          }
        } else {
          message.error(msg)
        }
      }
      if (sourceType === '站点') {
        getStation()
      }
    }
  }

  const tagRender = (props) => {

    const { label, value, closable, onClose } = props

    let color = options?.find(v => v.value === value)?.color || ''

    const onPreventMouseDown = (event) => {
      event.preventDefault()
      event.stopPropagation()
    }

    if (!color) {
      return label
    }

    return (
      <span style={{ marginRight: '3px' }}>
        <TagPack color={color} text={label} onClose={onClose} />
      </span>
    )
  }

  const isTagRender = color ? { tagRender } : {}

  return (
    <Select
      onFocus={onFocus}
      style={{ width: '100%' }}
      mode={isEdit ? 'tags' : (isMultiple ? 'multiple' : '')}
      disabled={disabled}
      showArrow={!disabled}
      showSearch={isSearch}
      value={valueShow}
      placeholder={disabled ? (placeholder || '') : (placeholder || '请选择内容')}
      onChange={handleChange}
      onSelect={onSelect}
      onDeselect={onDeselect}
      onClear={onClear}
      allowClear
      {...isTagRender}
    >
      {children}
    </Select>
  )

}

export default ComboBox
